JSON = 'json'
YAML = 'yaml'
TOML = 'toml'
NULL = 'null'
TRUE = 'true'
FALSE = 'false'
QUOTATION_MARK = '"'
MINUS = '-'
EXTRA_CHARACTERS_IN_LIST = [',', '\n', ' ']
EXTRA_CHARACTERS_IN_DICT = [',', '\n', ' ', ':']
EXTRA_ATTRIBUTE_CLASS_CODE=['__module__','__dict__','__weakref__']
